"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinarySearchTree = void 0;
class TreeNode {
    value;
    left;
    right;
    constructor(value) {
        this.value = value;
        this.left = null;
        this.right = null;
    }
}
class BinarySearchTree {
    root;
    comparator;
    constructor(comparator = (a, b) => a < b) {
        this.root = null;
        this.comparator = comparator;
    }
    /**
     * Inserts a value into the BST
     */
    insert(value) {
        const newNode = new TreeNode(value);
        if (!this.root) {
            this.root = newNode;
            return;
        }
        let current = this.root;
        while (true) {
            if (this.comparator(value, current.value)) {
                if (current.left === null) {
                    current.left = newNode;
                    break;
                }
                current = current.left;
            }
            else {
                if (current.right === null) {
                    current.right = newNode;
                    break;
                }
                current = current.right;
            }
        }
    }
    /**
     * Searches for a value in the BST. Returns true if found, false otherwise
     */
    find(value) {
        let current = this.root;
        while (current !== null) {
            if (this.isEqual(value, current.value)) {
                return true;
            }
            if (this.comparator(value, current.value)) {
                current = current.left;
            }
            else {
                current = current.right;
            }
        }
        return false;
    }
    /**
     * Returns the minimum value in the BST, or undefined if tree is empty
     */
    min() {
        if (!this.root)
            return undefined;
        let current = this.root;
        while (current.left !== null) {
            current = current.left;
        }
        return current.value;
    }
    /**
     * Returns the maximum value in the BST, or undefined if tree is empty
     */
    max() {
        if (!this.root)
            return undefined;
        let current = this.root;
        while (current.right !== null) {
            current = current.right;
        }
        return current.value;
    }
    /**
     * Removes a value from the BST if it exists
     */
    remove(value) {
        this.root = this.removeNode(this.root, value);
    }
    removeNode(node, value) {
        if (node === null)
            return null;
        if (this.comparator(value, node.value)) {
            node.left = this.removeNode(node.left, value);
            return node;
        }
        else if (this.comparator(node.value, value)) {
            node.right = this.removeNode(node.right, value);
            return node;
        }
        else {
            // Node to delete found
            // Case 1: Leaf node
            if (node.left === null && node.right === null) {
                return null;
            }
            // Case 2: Node with one child
            if (node.left === null)
                return node.right;
            if (node.right === null)
                return node.left;
            // Case 3: Node with two children
            const minNode = this.findMin(node.right);
            node.value = minNode.value;
            node.right = this.removeNode(node.right, minNode.value);
            return node;
        }
    }
    findMin(node) {
        let current = node;
        while (current.left !== null) {
            current = current.left;
        }
        return current;
    }
    isEqual(a, b) {
        // Two values are equal if neither is less than the other
        return !this.comparator(a, b) && !this.comparator(b, a);
    }
    /**
     * Executes a callback function for each element in the BST in-order traversal
     */
    forEach(callback, traversal = 'inorder') {
        switch (traversal) {
            case 'inorder':
                this.inorderTraversal(this.root, callback);
                break;
            case 'preorder':
                this.preorderTraversal(this.root, callback);
                break;
            case 'postorder':
                this.postorderTraversal(this.root, callback);
                break;
            default:
                this.inorderTraversal(this.root, callback);
        }
    }
    inorderTraversal(node, callback) {
        if (node === null)
            return;
        this.inorderTraversal(node.left, callback);
        callback(node.value);
        this.inorderTraversal(node.right, callback);
    }
    preorderTraversal(node, callback) {
        if (node === null)
            return;
        callback(node.value);
        this.preorderTraversal(node.left, callback);
        this.preorderTraversal(node.right, callback);
    }
    postorderTraversal(node, callback) {
        if (node === null)
            return;
        this.postorderTraversal(node.left, callback);
        this.postorderTraversal(node.right, callback);
        callback(node.value);
    }
    /**
     * Returns true if the BST is empty, false otherwise
     */
    isEmpty() {
        return this.root === null;
    }
    /**
     * Removes all nodes from the BST
     */
    clear() {
        this.root = null;
    }
    /**
     * Returns the number of nodes in the BST
     */
    count() {
        return this.countNodes(this.root);
    }
    countNodes(node) {
        if (node === null)
            return 0;
        return 1 + this.countNodes(node.left) + this.countNodes(node.right);
    }
}
exports.BinarySearchTree = BinarySearchTree;
//# sourceMappingURL=binary-search-tree.js.map